Package: Rpdb
Type: Package
Title: Read, Write, Visualize and Manipulate PDB Files
Version: 2.4.5
Date: 2026-05-11
Authors@R: 
	c(
    person("Leonard", "Mada", email = "lmada@umft.ro", role = c("cre", "ctb")),
    person("Julien", "Idé", role = c("aut"))
	)
Depends: rgl (>= 1.3.36), stats
Description: Provides tools to read, write, visualize Protein Data Bank (PDB) files and
    perform some structural manipulations.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
Repository: CRAN
URL: https://github.com/discoleo/Rpdb
BugReports: https://github.com/discoleo/Rpdb/issues
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-12 12:57:50 UTC; leo.mada
Author: Leonard Mada [cre, ctb],
  Julien Idé [aut]
Maintainer: Leonard Mada <lmada@umft.ro>
Date/Publication: 2026-05-12 16:20:11 UTC
Built: R 4.6.0; ; 2026-05-12 18:28:56 UTC; unix
