# Generated by roxygen2: do not edit by hand

S3method("[",atoms)
S3method("basis<-",default)
S3method("basis<-",pdb)
S3method("coords<-",atoms)
S3method("coords<-",pdb)
S3method(Mab,coords)
S3method(Mab,pdb)
S3method(Mbc,coords)
S3method(Mbc,pdb)
S3method(Mca,coords)
S3method(Mca,pdb)
S3method(Mxy,coords)
S3method(Mxy,pdb)
S3method(Myz,coords)
S3method(Myz,pdb)
S3method(Mzx,coords)
S3method(Mzx,pdb)
S3method(R,coords)
S3method(R,pdb)
S3method(Rx,coords)
S3method(Rx,pdb)
S3method(Ry,coords)
S3method(Ry,pdb)
S3method(Rz,coords)
S3method(Rz,pdb)
S3method(Ta,coords)
S3method(Ta,pdb)
S3method(Tabc,coords)
S3method(Tabc,pdb)
S3method(Tb,coords)
S3method(Tb,pdb)
S3method(Tc,coords)
S3method(Tc,pdb)
S3method(Tx,coords)
S3method(Tx,pdb)
S3method(Txyz,coords)
S3method(Txyz,pdb)
S3method(Ty,coords)
S3method(Ty,pdb)
S3method(Tz,coords)
S3method(Tz,pdb)
S3method(abc2xyz,atoms)
S3method(abc2xyz,coords)
S3method(abc2xyz,distances)
S3method(abc2xyz,pdb)
S3method(addEleLab,atoms)
S3method(addEleLab,pdb)
S3method(addResLab,atoms)
S3method(addResLab,pdb)
S3method(angle,coords)
S3method(angle,pdb)
S3method(atoms,atoms)
S3method(atoms,default)
S3method(atoms,pdb)
S3method(basis,default)
S3method(basis,pdb)
S3method(bond,coords)
S3method(bond,pdb)
S3method(cell.coords,crystal)
S3method(cell.coords,default)
S3method(cell.coords,pdb)
S3method(cell.density,default)
S3method(cell.density,pdb)
S3method(cell.volume,crystal)
S3method(cell.volume,pdb)
S3method(centres,atoms)
S3method(centres,coords)
S3method(centres,pdb)
S3method(centres,ppRoll)
S3method(centres.ppRoll,atoms)
S3method(centres.ppRoll,default)
S3method(centres.ppRoll,pdb)
S3method(chains,atoms)
S3method(chains,pdb)
S3method(connect,atoms)
S3method(connect,character)
S3method(connect,coords)
S3method(connect,default)
S3method(connect,nucleic)
S3method(connect,pdb)
S3method(connect.nucleic,atoms)
S3method(connect.nucleic,pdb)
S3method(coords,atoms)
S3method(coords,coords)
S3method(coords,data.frame)
S3method(coords,default)
S3method(coords,matrix)
S3method(coords,pdb)
S3method(crystal,default)
S3method(crystal,pdb)
S3method(dihedral,coords)
S3method(dihedral,pdb)
S3method(dist.point,default)
S3method(distances,atoms)
S3method(distances,connect)
S3method(distances,coords)
S3method(distances,default)
S3method(distances,pdb)
S3method(format.pdb.title,character)
S3method(inertia,atoms)
S3method(inertia,coords)
S3method(inertia,pdb)
S3method(info3d,atoms)
S3method(info3d,pdb)
S3method(lattice.range,pdb)
S3method(masses,default)
S3method(masses,pdb)
S3method(measure,coords)
S3method(measure,default)
S3method(measure,pdb)
S3method(merge,atoms)
S3method(merge,coords)
S3method(merge,pdb)
S3method(merge,pdb.hetero)
S3method(merge,pdb.resolution)
S3method(merge,pdb.structure)
S3method(mirror,coords)
S3method(mirror,pdb)
S3method(natom,atoms)
S3method(natom,coords)
S3method(natom,pdb)
S3method(norm,connect)
S3method(norm,distances)
S3method(pdb,default)
S3method(print,coords)
S3method(proj.line3d,matrix)
S3method(proj.line3d,numeric)
S3method(range,atoms)
S3method(range,coords)
S3method(range,pdb)
S3method(range.lattice,pdb)
S3method(reindex,atoms)
S3method(reindex,pdb)
S3method(replicate,atoms)
S3method(replicate,coords)
S3method(replicate,pdb)
S3method(select,atoms)
S3method(select,pdb)
S3method(split,pdb)
S3method(subset,atoms)
S3method(subset,pdb)
S3method(toSymbols,character)
S3method(toSymbols,integer)
S3method(toSymbols,numeric)
S3method(unsplit,default)
S3method(unsplit,pdb)
S3method(visualize,atoms)
S3method(visualize,character)
S3method(visualize,coords)
S3method(visualize,data.frame)
S3method(visualize,matrix)
S3method(visualize,pdb)
S3method(wrap,atoms)
S3method(wrap,coords)
S3method(wrap,pdb)
S3method(xyz2abc,atoms)
S3method(xyz2abc,coords)
S3method(xyz2abc,distances)
S3method(xyz2abc,pdb)
export("basis<-")
export("coords<-")
export(Mab)
export(Mbc)
export(Mca)
export(Mxy)
export(Myz)
export(Mzx)
export(R)
export(Rx)
export(Ry)
export(Rz)
export(Ta)
export(Tabc)
export(Tb)
export(Tc)
export(Tx)
export(Txyz)
export(Ty)
export(Tz)
export(abc2xyz)
export(addABC)
export(addBBox)
export(addEleLab)
export(addPBCBox)
export(addResLab)
export(addXYZ)
export(angle)
export(as.coords)
export(atoms)
export(basis)
export(bbox.pdb)
export(bond)
export(cell.coords)
export(cell.density)
export(cell.volume)
export(centres)
export(chains)
export(connect)
export(connect.nucleic)
export(coords)
export(crystal)
export(dihedral)
export(dist.point)
export(distances)
export(dotProd)
export(format.pdb.title)
export(inertia)
export(info3d)
export(is.atoms)
export(is.connect)
export(is.coords)
export(is.crystal)
export(is.distances)
export(is.pdb)
export(lattice.range)
export(masses)
export(measure)
export(mirror)
export(natom)
export(norm)
export(pdb)
export(proj.line3d)
export(range.lattice)
export(read.pdb)
export(reindex)
export(replicate)
export(rotVect)
export(select)
export(toSymbols)
export(trans)
export(translate)
export(unsplit)
export(vectNorm)
export(vectProd)
export(viewAB)
export(viewAxis)
export(viewBC)
export(viewCA)
export(viewInertia)
export(viewXY)
export(viewYZ)
export(viewZX)
export(visualize)
export(wrap)
export(write.pdb)
export(xyz2abc)
import(rgl)
importFrom(grDevices,rgb)
